/*
 * WaRP_DB_UART_Example.c
 *
 *  Created on: Mar 20, 2014
 *      Author: Jacob Postman
 *		Copyright: Revolution Robotics, Inc. 
 *		License: Createive Commons Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
 *
 *  Note: 
 *      Added JSON based messagnging and device handling and uart data handling. Reset of the code is based on FSL Qi charger handling (to be optimized)
 */

/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "Events.h"
#include "UART0.h"
#include "PTE.h"
#include "PTC.h"
#include "PTD.h"
#include "SM1.h"
#include "PTB.h"
#include "PWM1.h"
#include "TU1.h"
/* Including shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"

#include "WaRP_DB.h"
#include <common/common.h>
#include <uart.h>
#include <jsmn/jsmn.h>
#include <json/json.h>

#include "devices/device_MMA9553L.h"
#include "devices/device_Buzzer.h"

// FSL Power Receiver Defines
#include "derivative.h"
#include "prx_lib.h"
#include "params.h"
#include "defines.h"

#include "fsl_pwr_rcvr.h"


#define WARP_DB_FW_VER "0.2beta-20151012"

// Global UART IO Queues
CHAR_QUEUE UartRxQueue;
CHAR_QUEUE UartTxQueue;

// JSON Command Line Buffer 
CHAR_BUFFER UartLineBuffer;
char jsonStrBuffer[CHAR_BUFFER_SIZE];

/*
 *	Push Button 
 */
int debounceCounter; 

// WaRP Event Queue 
EVENT_QUEUE EventQueue;


/*
 * JSON Engine Queue
 */
JSON_ENGINE_QUEUE JsonEngineQueue;

char jsonEngineQueuePut(JSON_ENGINE_QUEUE *queue, jsonCmd_t json_cmd){
	// If queue is full, return an error
	if(queue->count == JSON_ENGINE_QUEUE_SIZE){
		return 1;
	}
	queue->count++;
	queue->queueArray[queue->in++] = json_cmd;
	if( queue->in >= JSON_ENGINE_QUEUE_SIZE){
		queue->in = 0;
	} 
	return 0; // No errors
}

char jsonEngineQueueGet(JSON_ENGINE_QUEUE *queue, jsonCmd_t *json_cmd){
	if(queue->count == 0){
		return 1;
	}
	queue->count--;
	*json_cmd = queue->queueArray[queue->out++];
	if( queue->out >= JSON_ENGINE_QUEUE_SIZE){
		queue->out = 0;
	}
	return 0; // No errors
}

void jsonEngineQueueInit(JSON_ENGINE_QUEUE *queue){
	queue->count = 0;
	queue->in = 0;
	queue->out = 0;
}

char eventQueuePut(EVENT_QUEUE *queue, warpEvent_t event){
	// If queue is full, return an error
	if(queue->count == EVENT_QUEUE_SIZE){
		return 1;
	}
	queue->count++;
	queue->queueArray[queue->in++] = event;
	if( queue->in >= EVENT_QUEUE_SIZE){
		queue->in = 0;
	} 
	return 0; // No errors
}

char eventQueueGet(EVENT_QUEUE *queue, warpEvent_t *event){
	if(queue->count == 0){
		return 1;
	}
	queue->count--;
	*event = queue->queueArray[queue->out++];
	if( queue->out >= EVENT_QUEUE_SIZE){
		queue->out = 0;
	}
	return 0; // No errors
}

void eventQueueInit(EVENT_QUEUE *queue){
	queue->count = 0;
	queue->in = 0;
	queue->out = 0;
}

char uartQueuePut(CHAR_QUEUE *queue, char item){
	// If queue is full, return an error
	if(queue->count == UART_QUEUE_SIZE){
		return 1;
	}
	queue->count++;
	queue->queueArray[queue->in++] = item;
	if( queue->in >= UART_QUEUE_SIZE){
		queue->in = 0;
	} 
	return 0; // No errors
}

char uartQueueGet(CHAR_QUEUE *queue, char *item){
	if(queue->count == 0){
		return 1;
	}
	queue->count--;
	*item = queue->queueArray[queue->out++];
	if( queue->out >= UART_QUEUE_SIZE){
		queue->out = 0;
	}
	return 0; // No errors
}

void uartQueueInit(CHAR_QUEUE *queue){
	queue->count = 0;
	queue->in = 0;
	queue->out = 0;
}

void printCharArray(unsigned char *charArray, unsigned char numBytes){
	unsigned char i;

	for(i=0; i<numBytes-1; i++){
		printf("0x%02X, ", charArray[i]);
	}
	printf("0x%02X", charArray[numBytes-1]);
}

int numDevices = 0;
device_t devices[10];

//void mmaHandler(void){
//	printf("MMA9553L Handler\n");
//}
//
//struct device_t MMA9553L_dev = {
//	.device_name = "MMA9553L",
//	.callback = &mmaHandler,
//};

int addDevice(device_t device){
	
	// Do not allow adding more devices than structure allows
	if(numDevices >= sizeof(devices)/sizeof(device_t))
			return -1;
	devices[numDevices].device_name = device.device_name;
	//devices[0]._device_index_id = device._device_index_id;
	devices[numDevices].callback = device.callback;
	
	numDevices++;
	
	return 0;
}

void runApplication(void){
	
	//unsigned char queryFWID[2] = {0x00, 0x00};
	
	char ch;
	
	// Initialize WaRP event queue
	eventQueueInit(&EventQueue);
	warpEvent_t warpEvent;
	debounceCounter = 0;
		
	// Initialize UART input queue
	uartQueueInit(&UartRxQueue);
	uartQueueInit(&UartTxQueue);
	
	// Initialize device drivers
	addDevice(MMA9553L_dev);
	addDevice(Buzzer_dev);
	/* add your own devices here */
		
	// Initialize JsonEngineQueue
	jsonEngineQueueInit(&JsonEngineQueue);
	jsonCmd_t jsonCmd;
	
	// Initialize line buffer
	UartLineBuffer.count = 0;
	
	// Announce daughterboard boot
	printf("{\"SYSMSG\":\"WaRP Daughterboard Started\",\"Version\":\"%s\"}\n", WARP_DB_FW_VER);
	
	int result;
	
	result = initMMA9553L();
	
	if( result < 0 ){
		printf("{\"SYSMSG\":\"MMA9553L init failed (%d).\"}\n", result);
	}
	
// ************** START FSL POWER RECEIVER INIT *************************************
	// FSL Power Receiver
	//float var_f, temp_f;
	init_system_variables();
	 
	// Calibrate ADC
	if (PE_AD1_Calibrate() != ERR_OK)
	{
		printf("\nADC0 Calibration Failed!\n");
		//while(1);  //debug - stop on fail
	}
	else
	{
		// Finish Calibration
		if (AdcLdd1_GetCalibrationResultStatus(NULL))
		{
			printf("\nADC0 Calibration Failed - halting board!\n");
			while(1); //debug - stop on fail
		}
	}
// ************** END FSL POWER RECEIVER INIT *************************************
	 
	while(1)
	{
		// Handle FSL Power Receiver
		if(detect_adapter()==1)
		{
			usb_charger_toggle = 1;
			if(getPkTxSts()==PK_TX_STOP)
			{
				// only when out of packet transmitting
				VUINT8 *pkBuf=g_pkTxCtrl.pkBuf;
				*pkBuf++=PK_HEADER_END_POWER;
				*pkBuf++=EPC_CHARGE_COMPLETE;
				prepare_packet_send();
			}      			  
		}
		qi_charging_state_machine();
		// End FSL Power Receiver main loop handling
		
		
		if(UartRxQueue.count){
			uartQueueGet(&UartRxQueue, &ch);
			
			if(
			(ch != 0x0D)	&& //CR
			(ch != 0x0A)	&& // LF/NL
			(UartLineBuffer.count < CHAR_BUFFER_SIZE))
			{
				switch (ch)
				{
				case 0x08:	// Backspace
				case 0x7F:	// Delete
					if (UartLineBuffer.count > 0){
						UartLineBuffer.count -= 1;
						out_char(0x08);	// Backspace
						out_char(' ');
						out_char(0x08);	// Backspace
					}
					break;
				default:
					if ((UartLineBuffer.count+2) < CHAR_BUFFER_SIZE){
						if ((ch > 0x1f) && (ch < 0x80)){
							UartLineBuffer.bufferArray[UartLineBuffer.count++] = ch;
							out_char(ch); // Echo - remove later
						}
					}
					break;
				}
			}
			else if (UartLineBuffer.count == CHAR_BUFFER_SIZE)	// Line length exceeds buffer size
			{
				// Send error and clear buffer
				UartLineBuffer.count = 0;
			}else // EOL detected - handle line
			{
				UartLineBuffer.bufferArray[UartLineBuffer.count] = '\0';
				out_char(0x0D);	// CR
				out_char(0x0A);	// LF
				//printf(UartLineBuffer.bufferArray);
				//out_char(0x0D);	// CR
				//out_char(0x0A);	// LF
				//strncpy(jsonStrBuffer, &UartLineBuffer.bufferArray[json_t[i].start], json_t[i].end - json_t[i].start);
				//jsonStrBuffer[json_t[i].end - json_t[i].start] = '\0';
				//printf(" JSON Token:\n  type: %s\n  start:%d\n  size:%d\n  end:%d\n  string:%s\n", 
				//	typeStr, json_t[i].start, json_t[i].size, json_t[i].end, jsonStrBuffer);
				jsonParseLine(UartLineBuffer.bufferArray);
				UartLineBuffer.count = 0;
			}
		}
		if(EventQueue.count){
			eventQueueGet(&EventQueue, &warpEvent);
			switch(warpEvent){
			case BTN0_DOWN:
				printf("{\"BTN0\":0}\n");
				break;
			case BTN0_UP:
				printf("{\"BTN0\":1}\n");
				break;
			case BTN1_DOWN:
				printf("{\"BTN1\":0}\n");
				break;
			case BTN1_UP:
				printf("{\"BTN1\":1}\n");
				break;
				// Get and send MMA9553L data
//				printf("{ \"MMA9553L\" : \"Status\", \"data\": {\"step count\" : 0, \"distance\" : 0, \"speed\" : 0, \"calories\" : 0} }\n");
			default:
				break;
			}
		}
		
		// device registration
		
		
		if(JsonEngineQueue.count){
			jsonEngineQueueGet(&JsonEngineQueue, &jsonCmd);
						
//			switch(jsonCmd.device){
//			case 1:
//				break;
//				
//			default:
//				// Error - unhandled device case.
//			}	
		}
		
		if(debounceCounter){
			debounceCounter--;
		}
	}
}

// Start PowerReceiver 

void init_HW(void)
{
        //Enable low voltage detect, COP.
        output_disable();

}

void init_system_variables(void)
{
        VUINT8 i;
        
        g_status=0;
        mtim_ms_cnt=0;
        mtim_0_1_s_cnt=0;
        mtim_sec_cnt=0;
        time_point_0_1_s=0;
        time_point_ms=0;
        g_systemCtrl.phase=QI_PHASE_SELECTION;
        //g_systemCtrl.phaseLast=QI_PHASE_SELECTION;
        g_systemCtrl.pkDelay=g_timing_Tbl[QI_TIMING_T_DELAY_SHORT];
        g_systemCtrl.ctrlErrCnt=g_SysParams.Trectified;;
        g_systemCtrl.isOutEnabled=0;
        rec_vol_expected=g_SysParams.rectified_voltage_expected;
#ifdef POWER_GOOD_PROTECTION
        dcdc_power_good=FALSE;
#endif
        //rec_under_voltage_cnt=0;
        //rec_vol_stable_cnt=0;
        //time_point_ms=0;

        /*Monitor*/
        rec_over_vol_cnt=0;
        output_short_cnt=0;
        output_disconnect_cnt=0;
        output_over_curr_cnt=0;
        output_over_vol_cnt=0;
        output_under_vol_cnt=0;
        end_power_code=0;
        system_error=0;
        system_runtime_status=0;

        rec_vol_buf_idx=0;
        for(i=0;i<ADC_AVERAGE_WINDOW_SIZE_REC_VOL;i++)
        {
                rec_vol_buf[i]=0;
        }

#ifdef ENABLE_OUT_CURR_SAMPLING_AVERAGE
        out_cur_10b_buf_idx=0;
        for(i=0;i<ADC_AVERAGE_WINDOW_SIZE_OUT_CURR;i++)
        {
                out_cur_10b_buf[i]=0;
        }
#endif

        init_com();

        /*debug data*/
#ifdef _DBG
        logBufCnt=0;
        TestPoint=0;
        d_pkTxCnt=0;
        d_byteTxCnt=0;
        d_bitTxCnt=0;
        d_bitClkCnt=0;
        d_testFlag=0;
#endif
}

void init_com(void)
{
        VUINT8 i;
        VUINT8 *ptr;

        outBufSts=0;
        bitClkFlag=0;
                
        ptr=(VUINT8 *)&g_pkTxCtrl;
        for(i=0;i<sizeof(g_pkTxCtrl);i++)
                *ptr++=0;
}

/*System monitor. Return error when system voltage/current/temperature is abnormal*/
VUINT8 system_monitor(void)
{
        if(getPkTxSts()==PK_TX_RUN)
            return 0;       /* If communication is on going, do not sampling because the signal would be mixed comm signal. */

        /* Do ADC sampling and system monitor every 1ms. */
        if(time_point_ms==mtim_ms_cnt)
            return 0;     
        else
            time_point_ms=mtim_ms_cnt;

#if (GPIO_DBG_PIN_1==GPIO_DBG1_MCU_WORKING)
        invert_dbg_gpio(IO_DBG1);
#endif

        /* System monitor*/
#ifdef ADC_10_BITS
        if(g_systemCtrl.DCon)
        {
            out_vol_16b=ADC_read_16b(ADC_CH_DC_OUT_VOLT);
            out_cur_16b=ADC_read_16b(ADC_CH_DC_OUT_CURR);
        }
        
        rec_vol_16b=ADC_read_16b(ADC_CH_VBUS);
        sys_temp_16b=ADC_read_16b(ADC_CH_TEMP);

#else
        
#ifdef ADC_10_BITS_FOR_RX_POWER
        
        if(g_systemCtrl.DCon)
		{
			out_vol_10b=ADC_read_16b(ADC_CH_DC_OUT_VOLT);
			out_cur_10b=ADC_read_16b(ADC_CH_DC_OUT_CURR);
			out_vol=(VUINT8)(out_vol_10b>>2);
			out_cur=(VUINT8)(out_cur_10b>>2);
		}
	
		rec_vol=(VUINT8)(ADC_read_16b(ADC_CH_VBUS)>>2);
		
#ifdef	INTERNAL_THERMAL	
        sys_temp=(VUINT8)(ADC_read_16b(ADC_CH_TEMP)>>2);
#endif
#ifdef	EXTERNAL_THERMAL
		ntc_thermistor_sample();
#endif
#else
        if(g_systemCtrl.DCon)
        {
            out_vol=ADC_read(ADC_CH_DC_OUT_VOLT);
            out_cur=ADC_read(ADC_CH_DC_OUT_CURR);
        }
        
        rec_vol=ADC_read(ADC_CH_VBUS);
#ifdef	INTERNAL_THERMAL	
        sys_temp=ADC_read(ADC_CH_TEMP);
#endif
#ifdef	EXTERNAL_THERMAL
        ntc_thermistor_sample();
#endif
#endif

#endif

#if 0
#ifdef POWER_GOOD_PROTECTION
        dcdc_power_good=testReg8Bits(PTCD,1<<IO_DC_PG)?TRUE:FALSE;
#endif
#endif
        ADC_average();

        if(time_point_0_1_s!=mtim_0_1_s_cnt)
        {       /*Do system error check every 0.1s.*/
                time_point_0_1_s=mtim_0_1_s_cnt;

                if(system_error<=SYS_ERR_OUT_DISCONNECT)
                        system_error=system_check();//system_error = 0;//system_error=system_check();

                if(system_error)
                {
#ifdef OUTPUT_PROTECTION
                        if(system_error==SYS_ERR_OUT_UNDER_VOLTAGE ||
                           system_error==SYS_ERR_OUT_OVER_VOLTAGE ||
                           system_error==SYS_ERR_OUT_SHORT)
                        {
                                output_disable();
                        }
#endif
                        
                        switch(system_error)
                        {
                                case SYS_ERR_OUT_DISCONNECT:
                                        end_power_code=EPC_CHARGE_COMPLETE;
                                        break;
                                case SYS_ERR_OUT_UNDER_VOLTAGE:
                                        end_power_code=EPC_INTERNAL_FAULT;
                                        break;
                                case SYS_ERR_OUT_SHORT:
                                        end_power_code=EPC_INTERNAL_FAULT;
                                        break;
                                case SYS_ERR_TEMP_OVER_HEAT:
                                        end_power_code=EPC_OVER_TEMPERATURE;
                                        break;
                                case SYS_ERR_OUT_OVER_VOLTAGE:
                                        end_power_code=EPC_OVER_VOLTAGE;
                                        break;
                                case SYS_ERR_IN_OVER_VOLTAGE:
                                        end_power_code=EPC_OVER_VOLTAGE;
                                        break;
                                case SYS_ERR_OUT_OVER_CURRENT:
                                        end_power_code=EPC_OVER_CURRENT;
                                        break;
                                default:
                                        end_power_code=EPC_UNKNOWN;
                                        break;
                        }

                }
        }

        return 0;
}

/*ADC sampling average.*/
/* PEx/Kinetis Code uses averaging built in.  May remove this later after more debug.... */
void ADC_average(void)
{
        VUINT8 i;
        VUINT16 sum;
        if(g_systemCtrl.phase<QI_PHASE_PING)
            return;

        /*rec_vol average*/
        rec_vol_buf[rec_vol_buf_idx]=rec_vol;
        rec_vol_buf_idx++;
        if(rec_vol_buf_idx>=ADC_AVERAGE_WINDOW_SIZE_REC_VOL)
        {
        
                rec_vol_buf_idx=0;
                for(i=0,sum=0;i<ADC_AVERAGE_WINDOW_SIZE_REC_VOL;i++)
                {
                        sum+=rec_vol_buf[i];
                }

                
#ifndef KINETIS_L_DEVICE
                /* RS08 original */
                //rec_vol_sum=sum;
                sum>>=3;	//shift right three is divide by 8 here.  RS08 original - updated to line below for Kinetis L based on header definition
#else
                sum = (sum / ADC_AVERAGE_WINDOW_SIZE_REC_VOL);
#endif
                rec_vol_average=(VUINT8)(sum&0xff);
        }

#ifdef ENABLE_OUT_CURR_SAMPLING_AVERAGE
        if(!g_systemCtrl.DCon)
            return;

        /*out_cur_10b average*/
        out_cur_10b_buf[out_cur_10b_buf_idx]=out_cur_10b;
        out_cur_10b_buf_idx++;
        if(out_cur_10b_buf_idx>=ADC_AVERAGE_WINDOW_SIZE_OUT_CURR)
        {
                out_cur_10b_buf_idx=0;
                for(i=0,sum=0;i<ADC_AVERAGE_WINDOW_SIZE_OUT_CURR;i++)
                {
                        sum+=out_cur_10b_buf[i];
                }

#ifndef KINETIS_L_DEVICE
                /* RS08 original */
                //rec_vol_sum=sum;
                sum>>=3;	//shift right three is divide by 8 here.  RS08 original - updated to line below for Kinetis L based on header definition
#else
                sum = (sum / ADC_AVERAGE_WINDOW_SIZE_OUT_CURR);
#endif
                out_cur_10b_average=sum;
        }
#endif

        /*out_vol average*/
        out_vol_buf[out_vol_buf_idx]=out_vol_10b;
        out_vol_buf_idx++;
        if(out_vol_buf_idx>=ADC_AVERAGE_WINDOW_SIZE_OUT_VOL)
        {
        
                out_vol_buf_idx=0;
                for(i=0,sum=0;i<ADC_AVERAGE_WINDOW_SIZE_OUT_VOL;i++)
                {
                        sum+=out_vol_buf[i];
                }

#ifndef KINETIS_L_DEVICE
                /* RS08 original */
                //rec_vol_sum=sum;
                sum>>=3;	//shift right three is divide by 8 here.  RS08 original - updated to line below for Kinetis L based on header definition
#else
                sum = (sum / ADC_AVERAGE_WINDOW_SIZE_OUT_VOL);
#endif
                out_vol_average = sum ;
        }

}

void read_current_sense_offset()
{
#ifdef ADC_10_BITS_FOR_RX_POWER
        curr_sense_offset=(VUINT8)ADC_read_16b(ADC_CH_DC_OUT_CURR);
#else
        curr_sense_offset=ADC_read(ADC_CH_DC_OUT_CURR);
#endif
}

/* Description: check if input and output voltage/current is in the range, check if output short happen.
 * Return: error status.*/
VUINT8 system_check(void)
{
        VUINT8 status;

        status=SYS_ERR_NONE;

        if(g_systemCtrl.phase!=QI_PHASE_PWR_TRANS)
                return status;
        
        
        /*over voltage check*/
        //if(rec_vol_average>g_SysParams.max_rec_voltage)
        if(rec_vol>g_SysParams.max_rec_voltage)        /*TODO: is ok to use instant voltage?*/
        {       
                system_runtime_status|=1<<SYS_ERR_IN_OVER_VOLTAGE;
                rec_over_vol_cnt++;
        }
        else
        {
                system_runtime_status&=~(1<<SYS_ERR_IN_OVER_VOLTAGE);
                rec_over_vol_cnt=0;
        }

#ifdef INPUT_OVER_VOLTAGE_CHECK
        if(rec_over_vol_cnt>=g_SysParams.time_out_of_rec_over_vol)
                return SYS_ERR_IN_OVER_VOLTAGE;
#endif

        if(g_systemCtrl.DCon)
        {
#ifdef POWER_GOOD_PROTECTION
                /*dc-dc power good signal is pulled low when output short or input under voltage(4.1v)*/
                if(!dcdc_power_good)
                {
                        output_short_cnt++;
                }
                else
                {
                        system_runtime_status&=~(1<<SYS_ERR_OUT_SHORT);
                        output_short_cnt=0;
                }
#endif

                /*under voltage check*/
                if(out_vol<g_SysParams.min_out_voltage)
                {       
                        system_runtime_status|=1<<SYS_ERR_OUT_UNDER_VOLTAGE;
                        output_under_vol_cnt++;
                }
                else
                {
                        system_runtime_status&=~(1<<SYS_ERR_OUT_UNDER_VOLTAGE);
                        output_under_vol_cnt=0;
                }
				
                /*over voltage check*/
                if(out_vol>g_SysParams.max_out_voltage)
                {       
                        system_runtime_status|=1<<SYS_ERR_OUT_OVER_VOLTAGE;
                        output_over_vol_cnt++;
                }
                else
                {
                        system_runtime_status&=~(1<<SYS_ERR_OUT_OVER_VOLTAGE);
                        output_over_vol_cnt=0;
                }
                /*output disconnect check*/
                if(out_cur_10b<(VUINT16)g_SysParams.min_out_current)
                {       
                        system_runtime_status|=1<<SYS_ERR_OUT_DISCONNECT;
                        output_disconnect_cnt++;      /*move to 1ms timer ISR*/
                }
                else
                {
                        system_runtime_status&=~(1<<SYS_ERR_OUT_DISCONNECT);
                        output_disconnect_cnt=0;
                }

                /*over current check*/
                if(out_cur_10b>(VUINT16)g_SysParams.max_out_current)
                {       
                        system_runtime_status|=1<<SYS_ERR_OUT_OVER_CURRENT;
                        output_over_curr_cnt++;
                }
                else
                {
                        system_runtime_status&=~(1<<SYS_ERR_OUT_OVER_CURRENT);
                        output_over_curr_cnt=0;
                }

                /*over temerature check, use NTC */
                if(sys_temp > (VUINT16)g_SysParams.max_temperture)
                {		
                        system_runtime_status|=1<<SYS_ERR_TEMP_OVER_HEAT;
                        output_over_temper_cnt++;
                }
                else
                {
                        system_runtime_status&=~(1<<SYS_ERR_TEMP_OVER_HEAT);
                        output_over_temper_cnt=0;
                }



#ifdef OUTPUT_SHORT_CHECK
                if(output_short_cnt>=g_SysParams.time_out_of_out_short)
                        status=SYS_ERR_OUT_SHORT;
#endif
#ifdef OUTPUT_OPEN_CHECK
                else if(output_disconnect_cnt>=g_SysParams.time_out_of_out_disconnect)
                        status=SYS_ERR_OUT_DISCONNECT;
#endif
#ifdef OUTPUT_OVER_VOLTAGE_CHECK
                else if(output_over_vol_cnt>=g_SysParams.time_out_of_out_vol_out_of_range)
                        status=SYS_ERR_OUT_OVER_VOLTAGE;
#endif
#ifdef OUTPUT_UNDER_VOLTAGE_CHECK
                else if(output_under_vol_cnt>=g_SysParams.time_out_of_out_vol_out_of_range)
                        status=SYS_ERR_OUT_UNDER_VOLTAGE;
#endif
#ifdef OUTPUT_OVER_CURRENT_CHECK
                else if(output_over_curr_cnt>=g_SysParams.time_out_of_out_over_curr)
                        status=SYS_ERR_OUT_OVER_CURRENT;
#endif
#ifdef OUTPUT_OVER_TEMPERATURE_CHECK
                else if(output_over_temper_cnt>=g_SysParams.time_out_of_out_over_temper)
                        status=SYS_ERR_TEMP_OVER_HEAT;
#endif

                else
                        status=SYS_ERR_NONE;
        }
        else
        {
                if(g_systemCtrl.isOutEnabled)
                        status=SYS_ERR_OUT_SHORT;
        }

        return status;
}

void wait_by_nop(VUINT8 count)
{
	VUINT8 i;
        
	for (i = 0; i < count; i++)     
		asm("NOP");
}

#ifdef KINETIS_L_DEVICE

void output_enable(void)
{
	/* assert DCDC_EN */
	DCDC_EN_ON;
	g_systemCtrl.DCon=1;
}

void output_disable(void)
{
	/* negate DCDC_EN */
	DCDC_EN_OFF;
	g_systemCtrl.DCon=0;
}

#else 

/* RS08 original */

void output_enable(void)
{
#if (HW_VERSION==HW_VERSION_1_1)
        setReg8Bits(PTBD,1<<IO_DC_EN); /*DC-DC on*/
#else if(HW_VERSION==HW_VERSION_1_2)
        clrReg8Bits(PTBD,1<<IO_DC_EN); /*DC-DC on*/
#endif
        g_systemCtrl.DCon=1;
}

void output_disable(void)
{
#if (HW_VERSION==HW_VERSION_1_1)
        clrReg8Bits(PTBD,1<<IO_DC_EN); /*DC-DC off*/
//#else if(HW_VERSION==HW_VERSION_1_2)
#elif(HW_VERSION==HW_VERSION_1_2)
        setReg8Bits(PTBD,1<<IO_DC_EN); /*DC-DC off*/
#endif
        g_systemCtrl.DCon=0;
}

#endif

#ifdef KINETIS_L_DEVICE

#if 0 /* moved to prx_lib.c */
void SET_TIMER1_INTERRUPT_TIME(VUINT32 time_in_us)
{
	VUINT32 timer_load_value;
	VUINT32 microseconds_per_tick;
		
	/* Bus Freq:  			CPU_BUS_CLK_HZ (original 24 MHz)
	 * 
	 * For 250 us delay:  24000000 * (250 / 1000000) => 24 * 250 => 6000 => 0x1770 ticks
	 */
	microseconds_per_tick = (CPU_BUS_CLK_HZ / 1000000);
	timer_load_value = (VUINT32)(time_in_us * microseconds_per_tick);	//calculate ticks required 
	
	/* first disable PIT */
	PIT_TCTRL1 = 0x00U;                  /* Clear control register to disable timer */
	PIT_TFLG1 = PIT_TFLG_TIF_MASK;       /* Clear timer flag register */
	
	/* Load new value for timer to count down */
	PIT_LDVAL1 = PIT_LDVAL_TSV(timer_load_value);
	
	/* Re-enable Timer and interrupt */
	PIT_TCTRL1 = (PIT_TCTRL_TIE_MASK | PIT_TCTRL_TEN_MASK); 
}
#endif

#else
void SET_MTIM2_TIME(VUINT8 length)
{
		/* RS08 Original - simply divided clock down to get the required delay */
        setReg8(MTIM2CLK,length);	//3=250us(div by 8), 5=1ms(div by 32)
	
		/* Set divider based on the delay required */
		
}
#endif /* #ifdef KINETIS_L_DEVICE */


/*

Add functions for rx pcb board version2

*/
#ifdef KINETIS_L_DEVICE
	
	/* Todo: add USB enable functionality if required */

#else

	/* RS08 Original */

void usb_enable(void)
{
        setReg8Bits(PTCD,1<<IO_USB_EN); /*usb_enable*/
}

void usb_disable(void)
{
        clrReg8Bits(PTCD,1<<IO_USB_EN); /*usb_disable*/
}

void rx_enable(void)
{

        setReg8Bits(PTCD,1<<IO_RX_EN); /*rx_enable*/

}

void rx_disable(void)
{
        clrReg8Bits(PTCD,1<<IO_RX_EN); /*rx_disable*/
}

#endif	

#ifdef KINETIS_L_DEVICE

/*
 * Custom function that uses Processor Expert Generated Code to measure ADC values
 * 
 * Channels being used are:
 * #define ADC_CH_DC_OUT_VOLT    		1   	//PTE16/ADC0_SE1 - Output DC voltage - ADC_V_WLREG_OUT 
 * #define ADC_CH_DC_OUT_CURR    		2   	//PTE18/ADC_SE2 - Output DC current - ADC_I_WLREG 
 * #define ADC_CH_VBUS   				5 		//PTE17/ADC_SE5a - Rectified voltage - ADC_VCOIL
 * 
 * Other functions called within ADC_read_16b
 * 
 * 	PE_AD1_Measure();
 * 	AD1_HWEnDi();
 * 	AD1_MainMeasure();
 * 	AdcLdd1_GetMeasuredValues();
  *	AdcLdd1_StartSingleMeasurement(NULL);
 *	AdcLdd1_CreateSampleGroup(LDD_TDeviceData *DeviceDataPtr, LDD_ADC_TSample *SampleGroupPtr, uint8_t SampleCount)
 *	AdcLdd1_GetMeasuredValues(LDD_TDeviceData *DeviceDataPtr, LDD_TData *BufferPtr)
 *	AdcLdd1_GetMeasurementCompleteStatus(LDD_TDeviceData *DeviceDataPtr)
 */
/************************************************************************************
 * Note:  For Kinetis L - the ADC_read_16b function reads all ADC channels
 * For better (future) optimization, change all calls of ADC_read_16b() to 
 * return a status flag instead of the results for that channel, and pass in 
 * a pointer to an array to hold the results of all channels instead.
 * Kind of like VUINT8 ADC_read_16b(word *results), where results == VUINT8 results[AD1_CHANNEL_COUNT]
 * Todo:  Update calls to ADC_read_16b
 *************************************************************************************/
VUINT16 ADC_read_16b(VUINT8 u8Channel)
{
	uint8_t i, adc_buffer_index;
	word averaged_adc_value[AD1_SAMPLE_GROUP_SIZE];
	
	/* Measure all channels using PEx auto-generated code */
	/* Sum of measurements will reside in AD1_OutV[x] where x=channel number */
	PE_AD1_Measure();	//->AD1_HWEnDi()->AD1_MainMeasure()->AdcLdd1_GetMeasuredValues()
		
	/* Average the measurements here
	 * Each AD1_OutV[x] element contains the sum of {AD1_SAMPLE_GROUP_SIZE} measurements for each x channel 
	 * Just divide it down to get the average
	 */
	for (i = 0; i < AD1_CHANNEL_COUNT; i++)
	{		
		/* calculate average for each channel.  This number is tied to PEx component - user beware of changing it!!! */
		/* Couldn't find a PEx parameter that defines the number of samples for ADC */
		/* would be nice to bring in the PEx constant for number of samples per channel but don't think it exists */
		/* User Beware:  If the number of samples is changed in processor expert, it must be updated here! */
		averaged_adc_value[i] = (AD1_OutV[i] / 4);		
	}		
	
	/* 
	 * ADC Channel number translation to buffer index 
	 * Return averaged value based on channel passed in 
	 */
	if (u8Channel == ADC_CH_DC_OUT_VOLT)
		adc_buffer_index = AdcLdd1_ADC_5VOUT_AD2;
	else if (u8Channel == ADC_CH_DC_OUT_CURR)
		adc_buffer_index = AdcLdd1_ADC_5VOUT_I_AD3;
	else if (u8Channel == ADC_CH_VBUS)
		adc_buffer_index = AdcLdd1_VBUS_AD1;
	else 
		adc_buffer_index = 0; /* default */
	
	return (averaged_adc_value[adc_buffer_index]);			
}

/*
 * Only returns 8-bit value
 */
VUINT8 ADC_read(VUINT8 u8Channel)
{
	return (VUINT8)(ADC_read_16b(u8Channel));	
}


#else

	/* RS08 original implementation */

VUINT8 ADC_read(VUINT8 u8Channel)
{
        ADCSC1 = u8Channel;
        while(!ADCSC1_COCO);
    
        return((VUINT8)ADCR);
}

VUINT16 ADC_read_16b(VUINT8 u8Channel)
{
        ADCSC1 = u8Channel;
        while(!ADCSC1_COCO);
    
        return((VUINT16)ADCR);
}

#endif /* #ifdef KINETIS_L_DEVICE */

/*Get MTIM counter */
VUINT8 get_time(void)
{
	      return mtim_ms_cnt;
}

#if 0
/*Description : Qi timing constrains check from current time point to certain time stamp.
* Input       : timId : timing constrains ID
                 : TSid : time stamp ID.
* Return      : false : time not reached, true: time out.*/ 
bool timing_check(VUINT8 timId, VUINT8 TSid)
{
        VUINT16 time, timing;


        time=CalcTimeLen(TSid);
        timing=(VUINT16)g_timing_Tbl[timId];

        if(time>timing)
                return TRUE;
        else
                return FALSE;
}

/*Record time stamp*/
void save_time_point(VUINT8 TP)
{
        if(TP<QI_TS_SIZE)
                time_point_buf[TP]=get_time();
}


/*Description : Get time stamp
* Input       : time point ID.
* Return      : time stamp */
VUINT16 get_time_point(VUINT8 TP)
{
        if(TP<QI_TS_SIZE)
                return time_point_buf[TP];
        else
                return 0;
}

/*Description : Calculate time length of certain time point till now.
* Input       : time point ID.
* Return      : time length(ms) */
VUINT16 CalcTimeLen(VUINT8 TP)
{
        VUINT16 start,end;

        start=get_time_point(TP);
        end=get_time();
        if(start<=end)
                return(end-start);
        else
                return(MTIM_MOD_VLAUE-(start-end));
}

/*delay function
 * Input: delay in ms*/
void delay_loop(VUINT16 delay)
{
        VUINT16 start, end, length;

        start=get_time();

        for(;;)
        {
                end=get_time();
                if(start<=end)
                        length=end-start;
                else
                        length=MTIM_MOD_VLAUE-(start-end);

                if(length>=delay)  
                        break;
        }
}
#endif

#ifdef KINETIS_L_DEVICE  

	/* Use Processor Expert BitIO LDD to assert/negate IO's */
	//GPIO_PDD_SetPortDataOutputMask(CTX_MODULE_BASE_ADDRESS, CTX_PORT_MASK);
	//GPIO_PDD_ClearPortDataOutputMask(CTX_MODULE_BASE_ADDRESS, CTX_PORT_MASK);
	
#else
/* removed for KL16 port */
void invert_dbg_gpio(VUINT8 io)
{
        invertReg8Bits(PTCD, 1<<io);
}

void set_dbg_gpio(VUINT8 io)
{
        setReg8Bits(PTCD, 1<<io);
}

void clr_dbg_gpio(VUINT8 io)
{
        clrReg8Bits(PTCD, 1<<io);
}

void set_GPIO(VUINT8 ID)
{
        VUINT8 mask;
        VUINT8 *GPIO_DR;
        
        GPIO_DR=get_GPIO_addr(ID,&mask);
        *GPIO_DR|=mask;
}

void clear_GPIO(VUINT8 ID)
{
        VUINT8 mask;
        VUINT8 *GPIO_DR;
        
        GPIO_DR=get_GPIO_addr(ID,&mask);
        *GPIO_DR&=~mask;
}
        
void invert_GPIO(VUINT8 ID)
{
        VUINT8 mask;
        VUINT8 *GPIO_DR;
        
        GPIO_DR=get_GPIO_addr(ID,&mask);
        *GPIO_DR^=mask;
}

/*Get GPIO data register address and GPIO mask.*/
VUINT8*  get_GPIO_addr(VUINT8 ID, VUINT8* mask)
{
        VUINT8 io;
        VUINT8 *GPIO_DR;
        
        io=GPIO_Table[ID];
        *mask=1<<(io&0xf);
        GPIO_DR=(VUINT8 *)0x20;
        GPIO_DR+=((io>>4)&0xf)<<1;
        return GPIO_DR;
}
#endif


#ifdef _DBG
#ifdef USE_PAGE_RAM_FOR_VARIABLES
void data_log(VUINT8 *__far data, VUINT8 TP, VUINT8 size)
#else
void data_log(VUINT8 *data, VUINT8 TP, VUINT8 size)
#endif
{
        VUINT8 i;

        if(TestPoint!=TP)
                return;
        
        for(i=0;i<size;i++)
        {
                logBuf[logBufCnt]=*data++;
                logBufCnt++;
                if(logBufCnt>=64)
                        logBufCnt=0;
        }
}
#endif



void ntc_thermistor_sample(void)
{

#ifdef KINETIS_L_DEVICE
	
	/* not using this adc channel - was tied to 3V3 on RS08 design */
	
#else
	
#ifdef ADC_10_BITS_FOR_RX_POWER
        sys_temp=(VUINT8)(ADC_read_16b(ADC_CH_ntc_thermistor)>>2);//PTB1/KBIP5/TXD/ADP5
#else
        sys_temp=ADC_read(ADC_CH_ntc_thermistor);//PTB1/KBIP5/TXD/ADP5
#endif
        
#endif /* KINETIS_L_DEVICE */

}
VUINT8 detect_adapter(void)
{
	
#ifdef KINETIS_L_DEVICE
	
	return 0; 	//usb/ac adapter detect, pin not used on KL
	
#else
	if (PTBD&(1<<IO_ADAPTOR_DETECT)) // check IO_ADAPTOR_DETECT input status
		return 1;
	else
		return 0;
#endif
	
}

//CTX
VUINT8 bitIO_read(void)
{
	//return testReg8Bits(PTBD,1<<IO_COM1);	//RS08
	return CTX_READ;
}

//CTX
void bitIO_clear(void)
{
    //clrReg8Bits(PTBD,1<<IO_COM1);			//RS08
	CTX_OFF;
}

//CTX
void bitIO_invert(void)
{
	//invertReg8Bits(PTBD, 1<<IO_COM1);		//RS08
	CTX_INVERT;
}

/*Bit send.
 * For bit "1", invert GPIO every 250us
 * For bit "0", invert GPIO every 500us
 * Input: bit to send
 * Return: bitClkFlag. 0: one bit clock end, next bit clock start.
 *                          1: half period of one bit clock  */
VUINT8 bit_send(VUINT8 Bit)
{
        if(Bit)
        {
        	//invertReg8Bits(PTBD, 1<<IO_COM1);	//RS08
        	CTX_INVERT;
        }
        else if(bitClkFlag)
        {
        	//invertReg8Bits(PTBD, 1<<IO_COM1);	//RS08
            CTX_INVERT;
            
#ifdef _DBG
                d_bitTxCnt++;
#endif
        }
                
        bitClkFlag^=1;	//exclusive OR -> 0^1 or 1^0 = TRUE, else FALSE (same=FALSE)

        return bitClkFlag;
}


// END PowerReceiver

