/*
 * device_MMA9553L.c
 *
 *  Created on: Sep 30, 2014
 *      Author: Jacob Postman
 *		Copyright: Revolution Robotics, Inc. 
 *		License: Createive Commons Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
 */
#include "device_MMA9553L.h"
#include "../WaRP_DB.h"

#include "Cpu.h"
#include "Events.h"
#include "UART0.h"
/* Including shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"

#include <common/common.h>
#include <uart.h>
#include <jsmn/jsmn.h>
#include <json/json.h>
#include <spi.h>




#ifdef MMA_DEBUG
#define mmaDbgPrintf(x)
#else
#define mmaDbgPrintf(x) printf(x)
#endif

#define FWID_RESPONSE_LEN				16
#define QUERYSLEEPWAKE_LEN 				4

#define PEDCFG_RESPONSE_LEN	 			20
#define PEDSTAT_RESPONSE_LEN 			16


const unsigned char REBOOT_TO_PED[]		= {0x29, 0x00, 0xFF, 0xFF, 0xFF, 0xFF};
const unsigned char RESETPED[] 			= {0x17, 0x20, 0x01, 0x01, 0x20};
const unsigned char QUERYFWID[] 		= {MMA955X_APP_VER, 0x00};
const unsigned char SLPWAK_WRITE[]		= {MMA955X_APP_SLPWAKE, 0x20, 0x06, 0x01, 0x00};
//const unsigned char QUERYSLEEPWAKE[]	= {MMA955X_APP_SLPWAKE, 0x80, 0x01, 0x01};

const unsigned char PEDCFG_WRITE[]		= 	{MMA955X_APP_PED, \
											0x20, 0x00, 0x10, 0x00, 0x00, 0x00, \
											0x00, 0x00, 0x01, 0x80, 0x00, 0xAF, \
											0x50, 0x04, 0x03, 0x05, 0x01, 0x00, 0x00};

const unsigned char PEDCFG_QUERY[]	= {MMA955X_APP_PED, 0x10, 0x00, 0x10};
const unsigned char PEDSTAT_QUERY[]	= {MMA955X_APP_PED, 0x30, 0x00, 0x0C};

unsigned char mmaReadBuffer[30] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
unsigned char mmaBuffer[30] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

int randValues[10] = {1, 3, 2, 4, 1, 5, 1, 2, 3, 1};
int randIndex = 0;
int tempsteps = 0;
int tempcalories = 0;
int tempdistance = 0;
int tempspeed = 0;

typedef struct {
	unsigned char appID;
	unsigned char COCO;
	unsigned char bytesRead;
	unsigned char bytesRequested;
	unsigned char pedStatRegMSB;
	unsigned char pedStatRegLSB;
	unsigned char stepCountMSB;
	unsigned char stepCountLSB;
	unsigned char distanceMSB;
	unsigned char distanceLSB;
	unsigned char speedMSB;
	unsigned char speedLSB;
	unsigned char caloriesMSB;
	unsigned char caloriesLSB;
	unsigned char sleepCountMSB;
	unsigned char sleepCountLSB;
} pedStatus_t;

pedStatus_t pedStatus;

#define getPedStatus(name) (pedStatus. name ## LSB)|((pedStatus. name ## MSB)<<8)


/*int mmaMultiByteReadSPI(unsigned char u8RegisterAddress, unsigned char *u8Buffer, char numBytes){
	
	MultiByteReadSPI(u8RegisterAddress, u8Buffer, numBytes);
	return 0;
}*/

int mmaCheckForError(void){
	if((pedStatus.COCO == 0x80) && (pedStatus.bytesRead == pedStatus.bytesRequested))
		return 0;
	else
		return -1;
}

int mmaCheckForDataReady(void){
	int result = MultiByteReadSPI(MAILBOX0, (unsigned char *)&pedStatus, 4);
	//printCharArray(pedStatus, 4);
	if( result < 0 )
		return -1;
	return mmaCheckForError();
}

int waitForPed(void){
	unsigned char retryCnt = 0;
	unsigned int delay;
	do{
		delay = 500000;
		while(--delay){}
		if (retryCnt++ == 20){
			jsonSysErr("Pedometer Error: time out.");
			return -1;
		}
	}while(mmaCheckForDataReady());
	return 0;
}

int MMA9553L_GetStatus(void){
	// Query Pedometer Status
	MultiByteWriteSPI(MAILBOX0, PEDSTAT_QUERY, sizeof(PEDSTAT_QUERY));
	if(waitForPed()) return -10;

	// Read Pedometer Status
	MultiByteReadSPI(MAILBOX0, (unsigned char *)&pedStatus, sizeof(pedStatus_t));
	//printCharArray(&pedStatus, sizeof(pedStatus_t));
	if(mmaCheckForError()) return -1;
	
	printf("{\"MMA9553L\":\"Status\",\"data\":{\"steps\":%d,\"distance\":%d,\"speed\":%d,\"calories\":%d}}\n",
			getPedStatus(stepCount),
			getPedStatus(distance),
			getPedStatus(speed),
			getPedStatus(calories));
	return 0;
}

void resetMMA()
{
	unsigned int i;
	GPIOD_PDOR &= ~0x10 ; // Set MMA_CS low
	GPIOC_PDOR &= ~0x80 ; // Force MMA reset low
	i=1000000;
	while(--i) { }
	GPIOC_PDOR |= 0x80 ;	// Release MMA reset
	i=1000000;
	while(--i) { }
	GPIOD_PDOR |= 0x10 ; // Set MMA_CS high
	i=5000000;
	while(--i) { }
}

int initMMA9553L(void)
{
	int result;
	int delaytime;
	int MMA9553LRebootCnt = 0;

	do{
		resetMMA();
		
		// Reboot to force into pedometer application code space
		result = MultiByteWriteSPI(MAILBOX0, REBOOT_TO_PED, sizeof(REBOOT_TO_PED));
		delaytime = 1000000;
		while(--delaytime){}
		
		// Request ped version
		result = MultiByteWriteSPI(MAILBOX0, QUERYFWID, sizeof(QUERYFWID));
		if(MMA9553LRebootCnt++ > 2) return -2;
	}while(waitForPed());
	
	result = MultiByteReadSPI(MAILBOX0, mmaReadBuffer, FWID_RESPONSE_LEN);
	if (result < 0) return -3;
	//printCharArray(mmaReadBuffer, FWID_RESPONSE_LEN);	
	// TODO: interpret version buffer response
	
	// Write SleepWake Config
	result = MultiByteWriteSPI(MAILBOX0, SLPWAK_WRITE, sizeof(SLPWAK_WRITE));
	if(waitForPed()) return -4;

	// Check SleepWake Config answer is okay
	result = MultiByteReadSPI(MAILBOX0,  mmaReadBuffer, 4);
	if (result < 0) return -5;

	// Write Pedometer Config
	result = MultiByteWriteSPI(MAILBOX0, PEDCFG_WRITE, sizeof(PEDCFG_WRITE));
	if(waitForPed()) return -6;

	// Check Pedometer Config answer is okay
	result = MultiByteReadSPI(MAILBOX0,  mmaReadBuffer, 4);
	if (result < 0) return -7;

	// Query Pedometer Config
	result = MultiByteWriteSPI(MAILBOX0, PEDCFG_QUERY, sizeof(PEDCFG_QUERY));
	if(waitForPed()) return -8;
	
	// Read Pedometer Config
	result = MultiByteReadSPI(MAILBOX0,  mmaReadBuffer, 20);
	if (result < 0) return -9;

	return MMA9553L_GetStatus();
}

void mmaHandler(char* jsonStr, jsmntok_t *jsmnTkns){
	
	int jsmnTknIndex, j, k, v;
	
	jsonParseState state = START;
	int objTknCnt = 0;
	int skipEndIndex = 0;
	int valFound = 0;
	
	//mmaDbgPrintf("MMA9553L Handler\n");
	//mmaDbgPrintf("String: %s\n", jsonStr);
	
	for(jsmnTknIndex=0, j=1; j > 0; jsmnTknIndex++, j--){
		jsmntok_t *t = &jsmnTkns[jsmnTknIndex];

		if (t->type == JSMN_ARRAY || t->type == JSMN_OBJECT)
			j += t->size;

		switch(state){
		case START:
			if (t->type != JSMN_OBJECT){
				jsonSysErr("JSON Error: root element must be an object.");
				return; //-1;
			}

			state = KEY;
			objTknCnt = t->size;

			if (objTknCnt == 0)
				state = STOP;

			if (objTknCnt % 2 != 0){
				jsonSysErr("JSON Error: object must have even number of children.");
				return; //-1;
			}
			break;
		case KEY:
			objTknCnt--;

			if(t->start <= skipEndIndex)
				break;

			if (t->type != JSMN_STRING){
				jsonSysErr("JSON Error: key elements must be strings.");
				return; //-1;
			}

			state = SKIP;

			for(k = 0; k < numDevices * sizeof(devices)/sizeof(device_t); k++){
				if(json_token_streq(jsonStr, t, devices[k].device_name)){
					state = VALUE;
					break;
				}
			}
			break;

		case SKIP:
			objTknCnt--;
			if(t->type != JSMN_STRING && t->type != JSMN_PRIMITIVE){
				// If the element to be skipped is an array or object, also skip it's children
				//token_index += t->size;
				if(t->end > skipEndIndex)
					skipEndIndex = t->end;
			}
			//mmaDbgPrintf("Skipping {key:value} pair: {\"%s\":\"%s\"}\n", 
					//json_token_tostr(jsonStr, &jsmnTkns[jsmnTknIndex-1]),
					//json_token_tostr(jsonStr, t));

			state = KEY;

			if (objTknCnt == 0)
				state = STOP;
			break;

		case VALUE:
					if (t->type != JSMN_STRING && t->type != JSMN_PRIMITIVE){
						jsonSysErr("JSON Error: value elements must be strings or primitives.");
						return; //-1;
					}
					//unsigned char retryCnt = 0;
					//char *subStr = json_token_tostr(jsonStr, t);
					json_token_tostr(jsonStr, t);
					//val_found = 0;
					//print("size of commands: %d", sizeof(MMA9553L_dev.commands));
					for (v = 0; v < sizeof(MMA9553L_dev.commands)/sizeof(char*); v++)
					{
						//printf("Searching for %s: %s\n", subStr, MMA9553L_dev.commands[v]);
						if (json_token_streq(jsonStr, t, MMA9553L_dev.commands[v]))
						{
							valFound = 1;
							break;
						}
					}
					if(valFound){

						switch(v){
							case(0):
								initMMA9553L();
								break;
							case(1):
								if(MMA9553L_GetStatus()) return;
								break;
							case(2):
								MultiByteReadSPI(MAILBOX0, mmaReadBuffer, sizeof(mmaReadBuffer));
								printf("{\"MMA9553L\":\"READMB\", \"data\":[");
								printCharArray(mmaReadBuffer, sizeof(mmaReadBuffer));
								printf("]\"}\n");
								break;
							case(3):
								MultiByteWriteSPI(MAILBOX0, QUERYFWID, sizeof(QUERYFWID));
								//printf("\n");
								//MultiByteReadSPI(MAILBOX0, mmaReadBuffer, FWID_RESPONSE_LEN);
								//printCharArray(mmaReadBuffer, FWID_RESPONSE_LEN);
								break;
							case(4):
								 // Reboot to force into pedometer application code space
								MultiByteWriteSPI(MAILBOX0, REBOOT_TO_PED, sizeof(REBOOT_TO_PED));
								break;
							case(5):
								 // toggle reset line
								resetMMA();
								break;
							default:
								break;
						}
					}else{
						//printf("{\"SYSMSG\":\"JSON Error: unknown JSON value: %s\"}\n", json_token_tostr(jsonStr, t));
						//return;
					}
				objTknCnt--;
				state = KEY;

				if (objTknCnt == 0)
					state = STOP;

				break;
		case STOP:
			// Just consume the tokens
			break;

		default:
			jsonSysErr("JSON Error: Invalid state.");
			return; //-1;
		}
	}
}

struct device_t MMA9553L_dev = {
	.device_name = "MMA9553L",
	.callback = mmaHandler,
	.commands =	{
			"INIT",		// 0
			"GET",		// 1
			"READMB",		// 2
			"QUERYFW",  // 3
			"REBOOT", 	// 4
			"RESET",	// 5
	}
};

