/*
 * i2c.c
 *
 *  Created on: Mar 21, 2014
 *      Author: Jacob Postman
 *		Copyright: Revolution Robotics, Inc. 
 *		License: Createive Commons Attribution-ShareAlike 4.0 International (CC BY-SA 4.0)
 *
 * Purpose:     Code for initializing and using I2C
 *
 * Notes:
 *
 */

/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "Events.h"
#include "UART0.h"

/* Including shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"

#include "i2c.h"
#include "common/common.h"

unsigned char SlaveID;

/*******************************************************************/
/*!
 * Pause Routine - 50 nop instructions
 */
void Pause(void){
	int n;
	for(n=1;n<50;n++) {
		asm("nop");
	}
}

/*******************************************************************/
/*!
 * Start I2C Transmision
 * @param SlaveID is the 7 bit Slave Address
 * @param Mode sets Read or Write Mode
 */
unsigned char IIC_StartTransmission (unsigned char slaveAddr, unsigned char Mode)
{
	// If Mode is not a valid value, return error. 
	if(Mode != MWSR && Mode != MRSW){
		return -1;
	}

	/* shift address in right possition */
	slaveAddr = (unsigned char) slaveAddr << 1;

	/* Set R/W bit at end of Slave Address */
	slaveAddr |= (unsigned char)Mode;

	/* send start signal */
	i2c_Start();

	/* send ID with W/R bit */
	i2c_write_byte(slaveAddr);
	
	return 0;
}


/*******************************************************************/
/*!
 * Read a register from the MMA device
 * @param u8RegisterAddress is Register Address
 * @return Data stored in Register
 */
unsigned char MMA9553LMultiByteRead(unsigned char u8RegisterAddress, unsigned char *u8readBuffer, char numBytes)
{
	//unsigned char result;
	unsigned char i;

	/* Send Slave Address */
	IIC_StartTransmission(MMA9553L_I2C_ADDRESS, MWSR);
	i2c_Wait();

	/* Write Register Address */
	I2C0_D = u8RegisterAddress;
	i2c_Wait();

	/* Do a repeated start */
	I2C0_C1 |= I2C_C1_RSTA_MASK;

	/* Send Slave Address */
	I2C0_D = (MMA9553L_I2C_ADDRESS << 1) | 0x01; //read address
	i2c_Wait();

	/* Put in Rx Mode */
	I2C0_C1 &= (~I2C_C1_TX_MASK);

	// Dummy read, buffer[0] will be overwritten in loop that follows
	u8readBuffer[0] = I2C0_D; 
	//for (j=0; j<5000; j++){}; // ??? Silly example code
	i2c_Wait();
		
	for (i=0; i<numBytes-1; i++){
		u8readBuffer[i] = I2C0_D;
		i2c_Wait();
	}
	
	// Turn off ACK
	I2C0_C1 |= I2C_C1_TXAK_MASK;

	// Read last byte
	u8readBuffer[numBytes-1] = I2C0_D ;
	//for (j=0; j<5000; j++){};
	i2c_Wait();

	
	/* Send stop */
	i2c_Stop();
	//result = I2C1_D ;
	Pause();
	
	// Turn on ACK
	I2C0_C1 &= ~I2C_C1_TXAK_MASK;
	
	return 0;
}

/*******************************************************************/
/*!
 * Write a byte of Data to the MMA7660
 * @param u8RegisterAddress is Register Address
 * @param u8Data is Data to write
 */
unsigned char MMA9553LMultiByteWrite(unsigned char u8RegisterAddress, unsigned char *u8readBuffer, char numBytes)
{
	unsigned char i;

	/* send data to slave */
	IIC_StartTransmission(MMA9553L_I2C_ADDRESS, MWSR);
	i2c_Wait();

	/* Send I2C address */
	I2C0_D = u8RegisterAddress;
	i2c_Wait();

	for(i=0; i<numBytes; i++)
	{
		/* Send data */
		I2C0_D = u8readBuffer[i];
		i2c_Wait();  
	}

	i2c_Stop();

	Pause();
	return 0;
}
